@@ -220,15 +220,11 @@ class Interpreter:
     def raise_exception(self, node, exc=None, msg='', expr=None,
                         lineno=None):
         """Add an exception."""
-        if self.error is None:
-            self.error = []
-        if len(self.error) > 1:
-            return
         if expr is None:
             expr = self.expr
-        if len(self.error) > 0 and not isinstance(node, ast.Module):
-            msg = f'{msg!s}'
+        msg = str(msg)
         err = ExceptionHolder(node, exc=exc, msg=msg, expr=expr, lineno=lineno)
+
         self._interrupt = ast.Raise()
         self.error.append(err)
         if self.error_msg is None:
@@ -236,10 +232,13 @@ class Interpreter:
         elif len(msg) > 0:
             self.error_msg = f"{exc:s}: {msg}"
         if exc is None:
-            try:
-                exc = self.error[0].exc
-            except:
-                exc = RuntimeError
+            exc = self.error[-1].exc
+            if exc is None and len(self.error) > 0:
+                while exc is None and len(self.error) > 0:
+                    err = self.error.pop()
+                    exc = err.exc
+        if exc is None:
+            exc = Exception
         raise exc(self.error_msg)
 
     # main entry point for Ast node evaluation
@@ -290,7 +289,6 @@ class Interpreter:
 
         # run the handler:  this will likely generate
         # recursive calls into this run method.
-
         try:
             ret = handler(node)
             if isinstance(ret, enumerate):
@@ -300,6 +298,15 @@ class Interpreter:
             if with_raise:
                 self.raise_exception(node, expr=expr)
                 raise
+
+        # avoid too many repeated error messages (yes, this needs to be "2")
+        if len(self.error) > 2:
+            error = [self.error[0]]
+            for err in self.error[1:]:
+                le = error[-1]
+                if err.exc != le.exc or err.expr != le.expr or err.msg !=  le.msg:
+                    error.append(err)
+            self.error = error
         return None
 
     def __call__(self, expr, **kw):
@@ -317,30 +324,23 @@ class Interpreter:
             except Exception:
                 errmsg = exc_info()[1]
                 if len(self.error) > 0:
-                    errmsg = self.error[0].get_error()[1]
-                if raise_errors:
-                    try:
-                        exc = self.error[0].exc
-                    except Exception:
-                        exc = RuntimeError
-                    raise exc(errmsg)
+                    lerr = self.error[-1]
+                    errmsg = lerr.get_error()[1]
+                    if raise_errors:
+                        raise lerr.exc(errmsg) from exc
                 if show_errors:
                     print(errmsg, file=self.err_writer)
                 return None
         else:
             node = expr
         try:
-            return self.run(node, expr=expr, lineno=lineno)
+            return self.run(node, expr=expr, lineno=lineno, with_raise=raise_errors)
         except:
             errmsg = exc_info()[1]
             if len(self.error) > 0:
-                errmsg = self.error[0].get_error()[1]
+                errmsg = self.error[-1].get_error()[1]
             if raise_errors:
-                try:
-                    exc = self.error[0].exc
-                except Exception:
-                    exc = RuntimeError
-                raise exc(errmsg)
+                raise self.error[-1].exc(errmsg) from exc
             if show_errors:
                 print(errmsg, file=self.err_writer)
         return None
@@ -496,8 +496,7 @@ class Interpreter:
         fmt = '{__fstring__}'
         if node.format_spec is not None:
             fmt = f'{{__fstring__:{self.run(node.format_spec)}}}'
-        else:
-            return fmt.format(__fstring__=val)
+        return fmt.format(__fstring__=val)
 
     def _getsym(self, node):
         val = self.symtable.get(node.id, ReturnedNone)
@@ -598,16 +597,10 @@ class Interpreter:
         """Extended slice."""
         return tuple([self.run(tnode) for tnode in node.dims])
 
-    def on_subscript(self, node):    # ('value', 'slice', 'ctx')
-        """Subscript handling -- one of the tricky parts."""
-        val = self.run(node.value)
-        nslice = self.run(node.slice)
-        ctx = node.ctx.__class__
-        if ctx in (ast.Load, ast.Store):
-            return val[nslice]
-        msg = "subscript with unknown context"
-        self.raise_exception(node, msg=msg)
-        return None
+    def on_subscript(self, node): # ('value', 'slice', 'ctx')
+        """Subscript handling"""
+        return self.run(node.value)[self.run(node.slice)]
+
 
     def on_delete(self, node):    # ('targets',)
         """Delete statement."""
@@ -623,9 +616,27 @@ class Interpreter:
                 children.append(tnode.id)
                 children.reverse()
                 self.symtable.pop('.'.join(children))
-            else:
-                msg = "could not delete symbol"
-                self.raise_exception(node, msg=msg)
+            elif tnode.__class__ == ast.Subscript:
+                nslice = self.run(tnode.slice)
+                children = []
+                tnode = tnode.value
+                while tnode.__class__ == ast.Attribute:
+                    children.append(tnode.attr)
+                    tnode = tnode.value
+                if (tnode.__class__ == ast.Name and not
+                    tnode.id in self.readonly_symbols):
+                    children.append(tnode.id)
+                    children.reverse()
+                    sname = '.'.join(children)
+                    val = self.run(sname)
+                    del val[nslice]
+                    if len(children) == 1:
+                        self.symtable[sname] = val
+                    else:
+                        child = self.symtable[children[0]]
+                        for cname in children[1:-1]:
+                            child = child[cname]
+                        setattr(child, children[-1], val)
 
     def on_unaryop(self, node):    # ('op', 'operand')
         """Unary operator."""
@@ -651,23 +662,18 @@ class Interpreter:
         """comparison operators, including chained comparisons (a<b<c)"""
         lval = self.run(node.left)
         results = []
+        multi = len(node.ops) > 1
         for oper, rnode in zip(node.ops, node.comparators):
             rval = self.run(rnode)
             ret = op2func(oper)(lval, rval)
-            results.append(ret)
-            try:
-                if not ret:
-                    break
-            except ValueError:
-                pass
-            lval = rval
-        if len(results) == 1:
-            out = results[0]
-        else:
-            out = True
-            for ret in results:
-                out = out and ret
-        return out
+            if multi:
+                results.append(ret)
+                if not all(results):
+                    return False
+                lval = rval
+        if multi:
+            ret = all(results)
+        return ret
 
     def _printer(self, *out, **kws):
         """Generic print function."""
@@ -761,7 +767,6 @@ class Interpreter:
                     saved_syms[tnode.target.id] = copy.deepcopy(self._getsym(tnode.target))
 
             elif tnode.target.__class__ == ast.Tuple:
-                target = []
                 for tval in tnode.target.elts:
                     if tval.id in self.symtable:
                         saved_syms[tval.id] = copy.deepcopy(self._getsym(tval))
@@ -769,8 +774,10 @@ class Interpreter:
 
 
     def do_generator(self, gnodes, node, out):
+        """general purpose generator """
         gnode = gnodes[0]
         nametype = True
+        target = None
         if gnode.target.__class__ == ast.Name:
             if (not valid_symbol_name(gnode.target.id) or
                 gnode.target.id in self.readonly_symbols):
@@ -782,7 +789,7 @@ class Interpreter:
             target = tuple([gval.id for gval in gnode.target.elts])
 
         for val in self.run(gnode.iter):
-            if nametype:
+            if nametype and target is not None:
                 self.symtable[target] = val
             else:
                 for telem, tval in zip(target, val):
