@@ -478,6 +478,26 @@ def test_cmp(nested):
     isfalse(interp, "3 > 5")
     isfalse(interp, "5 < 3")
 
+@pytest.mark.parametrize("nested", [False, True])
+def test_comparisons_return(nested):
+    """test comparisons that do not return a bool"""
+    interp = make_interpreter(nested_symtable=nested)
+    if HAS_NUMPY:
+
+        x = np.arange(10)/1.2
+        out = x > 2.3
+
+        interp("x = arange(10)/1.2")
+        interp("out = x > 2.3")
+
+        assert all(interp.symtable['out'] == out)
+        assert interp.symtable['out'].sum() == 7
+
+        interp("out = (x > 2.3 < 6.2)")
+
+        assert interp.error.pop().exc == ValueError
+
+
 @pytest.mark.parametrize("nested", [False, True])
 def test_bool(nested):
     """boolean logic"""
@@ -1485,6 +1505,47 @@ def test_stringio(nested):
     intrep("print('out')")
     assert out.getvalue() == 'out\n'
 
+@pytest.mark.parametrize("nested", [False, True])
+def test_gh129(nested):
+    """ test that errors are propagated correctly, GH #129
+    """
+    interp = make_interpreter(nested_symtable=nested)
+    interp('one, two, default  = 1, 2, 3')
+    interp(textwrap.dedent("""
+    try:
+        output = some_var
+    except NameError:
+        output = None
+    foo = output or default
+    """))
+    assert len(interp.error) == 0
+    assert interp('foo') == 3
+
+@pytest.mark.parametrize("nested", [False, True])
+def test_no_duplicate_exception(nested):
+    """ test that errors are not repeated GH #132
+    """
+    interp = make_interpreter(nested_symtable=nested)
+    interp.run("print(hi)", with_raise = False)
+    assert len(interp.error) == 1
+    assert interp.error[0].exc == NameError
+
+
+@pytest.mark.parametrize("nested", [False, True])
+def test_delete_slice(nested):
+    """ test that a slice can be deleted"""
+
+    interp = make_interpreter(nested_symtable=nested)
+    interp.run("dlist = [1,3,5,7,9,11,13,15,17,19,21]")
+    interp.run("del dlist[4:7]")
+    assert interp("dlist") == [1, 3, 5, 7, 15, 17, 19, 21]
+
+    if nested:
+        interp.run("g = Group()")
+        interp.run("g.dlist = [1,3,5,7,9,11,13,15,17,19,21]")
+        interp.run("del g.dlist[4:7]")
+        assert interp("g.dlist") == [1, 3, 5, 7, 15, 17, 19, 21]
+
 
 if __name__ == '__main__':
     pytest.main(['-v', '-x', '-s'])
